#include "ubl-settings-manager.h"

app_section *yon_app_section_new(){
    app_section *new_section = malloc(sizeof(app_section));
    memset(new_section,0,sizeof(app_section));
    return new_section;
}